/*
The new TIM2TGA converts a PSX-TIM file into a RGB-Targa file.
Now it works ! I fixed the color-system, and a small bug.

You need a C-compiler to compile it. I tried to get this simple sourcecode
CPU, OS and compiler independent. Your system should offer flat memory
("malloc" a block >64K !).

You need a dpmi-server, for example cwsdpmi.exe, or a the DOS-box from
windows or OS/2.

You should convert the TGA to jpeg to save disk-space.

For comments, suggestions or questions write to: 
Jens Ch. Restemeier <jchrr@hrz.uni-bielefeld.de>

Many thanks go to:
Davide Pasca <dpasca@ix.netcom.com>
because he fixed the bug with the color-system !

					Jens Ch. Restemeier
*/

#include <stdio.h>
#include <stdlib.h>

/* picture buffer */
int xsize=0,ysize=0;
char *scr=NULL;

/* read/write buffer */
unsigned char buffer[2048];

int load_tim(char *name)
{
  FILE *timfile;
  int rc=1;
  if ((timfile=fopen(name,"rb"))!=NULL) {
    char *p;
    fread(&buffer,20,1,timfile);
    /* All TIM-files I know begin with 0x10. If not, kill this line ! */
    if (buffer[0]==0x10) {
      xsize=buffer[16]|(buffer[17]<<8);
      ysize=buffer[18]|(buffer[19]<<8);
      if ((xsize*2) < sizeof(buffer)) {
	printf("size: %u x %u\n",xsize,ysize);
        if ((p=scr=malloc(xsize*ysize*3))!=NULL) {
	  int x,y;
	  fseek(timfile,20,SEEK_SET);
	  for (y=0;y<ysize;y++) {
	    if (!feof(timfile)) {
	      fread(buffer,2,xsize,timfile);
	      for (x=0;x<xsize;x++) {
                unsigned short pixel, r, g, b;
                /* convert 2 byte from the buffer into one word */
                pixel=buffer[x<<1]|(buffer[(x<<1)+1]<<8);

                /* "b" needs "& 31" as well to strip 16th bit used for semi-transparency */
                b = (pixel >> 10) & 31;
                g = (pixel >>  5) & 31;
                r = (pixel      ) & 31;

                /* assuming TGA format is BGR */
                *p++ = b << 3;
                *p++ = g << 3;
                *p++ = r << 3;
	      }
	    }
	  }
	}
	else {
	  printf("error allocating memory !\n");
	  rc=0;
	}
      } else {
	printf("buffer too small to read a line !\n");
	rc=0;
      }
    } else {
       printf("unknown format !\n");
       rc=0;
    }
    fclose (timfile);
  } else {
    printf("error reading %s !\n",name);
    rc=0;
  }
  return rc;
}

int save_tga(char *name)
{
  FILE *tgafile;
  if ((tgafile=fopen(name,"wb"))!=NULL) {
    int c,i,j,k;
    /* create Targa-header */
    buffer[0]=0;                /* No image-ID field */
    buffer[1]=0;                /* no color map */
    buffer[2]=2;                /* rgb-pic uncompressed */
    buffer[3]=0;buffer[4]=0;    /* color map offset */
    buffer[5]=0;buffer[6]=0;    /* 0 entrys */
    buffer[7]=24;               /* 24 Bit color */
    buffer[8]=0;buffer[9]=0;    /* X-Position */
    buffer[10]=0;buffer[11]=0;  /* Y-Position */
    buffer[13]=(xsize>>8)&255;buffer[12]=xsize&255;     /* X-Size */
    buffer[15]=(ysize>>8)&255;buffer[14]=ysize&255;     /* Y-Size */
    buffer[16]=24; buffer[17]=0; /* 24 bit Color */
    fwrite(&buffer,18,1,tgafile);
    /* save bitmap (bottom up) */
    for (c=1;c<=ysize;c++)
      fwrite(scr+((ysize-c)*xsize*3),xsize*3,1,tgafile);
    fclose(tgafile);
    return 1;
  } else {
    printf("error writing %s !\n",name);
  }
  return 0;
}

int main(int argc,char *argv[])
{
  if (argc==3) {
     if (load_tim(argv[1])) {
       if (save_tga(argv[2]))
	 printf("done.\n");
       free(scr);
     }
  } else
      printf("TIM2TGA <infile> <outfile>\n");
  return 0;
}
